package FiguryWersja03;

import java.util.Scanner;

abstract class Figura
{
    abstract public String podajNazwe();
    abstract public double pole();
}

class Kwadrat extends Figura
{
    public Kwadrat(double bok) { this.bok = bok; }
    public Kwadrat() { this(0); }
    @Override
    public double pole() { return bok * bok; }
    public void ustawBok(double bok) { this.bok = bok; }
    public double pobierzBok() { return bok; }
    @Override
    public String podajNazwe() { return "kwadrat"; }    
    private double bok;
}

class Kolo extends Figura
{
    public Kolo(double promien) { this.promien = promien; }
    public Kolo() { this(0); }
    @Override
    public double pole() { return Math.PI * promien * promien; }
    public void ustawPromien(double promien) { this.promien = promien; }
    public double pobierzPromien() { return promien; }
    @Override
    public String podajNazwe() { return "koło"; }  
    private double promien;
}
abstract class AbstrakcyjnaFabrykaFigur
{
    abstract public Figura utworzFigure(int kodFigury);
}

class PrawidlowaFabrykaFigur extends AbstrakcyjnaFabrykaFigur
{
    public Figura utworzFigure(int kodFigury)
    {
        switch(kodFigury)
        {
            case 1  : return new Kwadrat();
            case 2  : return new Kolo();
            default : return null;
        }
    }
}

class CyganiacaFabrykaFigur  extends AbstrakcyjnaFabrykaFigur
{
     @Override
     public Figura utworzFigure(int kodFigury)
    {
        switch(kodFigury)
        {
            case 1  : return new Kolo();
            case 2  : return new Kwadrat();
            default : return null;
        }
    }
}

interface OperacjeWejsciaWyjscia
{
    void piszKomunikatWstepny();
    int ustalKodFigury();
    void wypiszWynik(String nazwa, double pole);
    void pokazMenu();
    double czytajLiczbeRzeczywista(String komunikat);
}

class Konsola implements OperacjeWejsciaWyjscia
{
    @Override
    public void piszKomunikatWstepny()
    {
        System.out.println("Obliczam pola figur, proszę wybrać figurę");    
    }
    @Override
    public int ustalKodFigury()
    {
        pokazMenu();
        return wejscie.nextInt();
    }
    @Override
    public void wypiszWynik(String nazwa, double pole)
    {
        System.out.println( "Pole figury " + nazwa + " wynosi " + pole);
    }
    @Override
    public void pokazMenu()
    {
        System.out.print("\n1. Kwadrat\n2. Koło\n0. Koniec\n>> ");
    }
    @Override
    public double czytajLiczbeRzeczywista(String komunikat)
    {
        System.out.print(komunikat);
        return wejscie.nextDouble();
    }
    private static Scanner wejscie = new Scanner(System.in);
}


class KalkulatorPol        
{
    KalkulatorPol(OperacjeWejsciaWyjscia interfejs)
    {
        this.interfejs = interfejs;
    }
    public void ustalParametryFigury(Figura figura)
    {
        if( figura instanceof Kwadrat )
        {
            double bok;
            bok = interfejs.czytajLiczbeRzeczywista("Podaj długość boku: ");
            ((Kwadrat)figura).ustawBok(bok);
        }
        if( figura instanceof Kolo )
        {
            double promien;
            promien = interfejs.czytajLiczbeRzeczywista("Podaj długość promienia: ");
            ((Kolo)figura).ustawPromien(promien);
        }    
    }
    
    public void wykonanie(AbstrakcyjnaFabrykaFigur fabryka)
    {
        int kodFigury;
        
        interfejs.piszKomunikatWstepny();
        Figura figura;
        do
        {
            kodFigury = interfejs.ustalKodFigury();
            if((figura = fabryka.utworzFigure(kodFigury)) != null)
            {
                ustalParametryFigury(figura);
                interfejs.wypiszWynik(figura.podajNazwe(), figura.pole()); 
            }
        }
        while(kodFigury != 0);
    }
    OperacjeWejsciaWyjscia interfejs;
}

public class Figury03 
{
    public static void main(String[] args) 
    {
        KalkulatorPol kalk = new KalkulatorPol(new Konsola());
        
        kalk.wykonanie(new PrawidlowaFabrykaFigur());
                
    }
}